// **************************************************
// SerialLib.cpp
//
//	Author: Marcio Serolli Pinho 
//	pinho@inf.pucrs.br
// **************************************************


#include <stdio.h>
#include <windows.h>


#include "SerialLib.h"



HANDLE handle;


// ***************************************************
//
// ***************************************************
int OpenSerialPort(char *port, int speed, int readTimeout)
{

    DCB SerialSettings;
    COMMTIMEOUTS timeOut;

    handle = CreateFile(port,   
                       GENERIC_READ | GENERIC_WRITE, 
                       0, 0, OPEN_EXISTING,
                       FILE_ATTRIBUTE_NORMAL, NULL);
    if (handle == INVALID_HANDLE_VALUE) {
        fprintf(stderr, "Erro abrindo a porta '%s'\n",
                port);
        return -1;
	}

    FillMemory(&SerialSettings, sizeof(SerialSettings), 0);
    if (!GetCommState(handle, &SerialSettings)) {
        fprintf(stderr, "Erro !!\n");
        CloseHandle(handle);
        return -1;
    }

    SerialSettings.BaudRate = speed;
    SerialSettings.ByteSize = 8;
    SerialSettings.fParity = FALSE;
    SerialSettings.Parity = NOPARITY;
	SerialSettings.StopBits = ONESTOPBIT;

    if (!SetCommState(handle, &SerialSettings)) {
        fprintf(stderr, "Erro setando configuraes da porta !\n");
        CloseHandle(handle);
        return -1;
    }

    timeOut.WriteTotalTimeoutConstant = 1000; /* 1 sec */
    timeOut.WriteTotalTimeoutMultiplier = 0;
    timeOut.ReadIntervalTimeout = 0;
    timeOut.ReadTotalTimeoutConstant = readTimeout;

    timeOut.ReadTotalTimeoutMultiplier = 1;

    if (!SetCommTimeouts(handle, &timeOut)) {
        fprintf(stderr, "Erro setando timeout!!\n");
        return(0);
    }

    return ((int)handle);
}


// ***************************************************
//
// ***************************************************
void CloseSerialPort()
{
   if (!CloseHandle(handle)) 
      fprintf(stderr, "Problems closing port\n");
}
// ***************************************************
//
// ***************************************************
int WriteSerialOneByte (int OneByte)
{

	DWORD NumberOfBytesSent;


    if (!WriteFile(handle, &OneByte, 1, &NumberOfBytesSent, NULL)) {
        return(0);
    }

    if (NumberOfBytesSent != 1) {
        fprintf(stderr, "Problemas com a escrita na porta serial.\n");
        return(0);
    }

    return(1);
}

// ***************************************************
//
// ***************************************************
int WriteSerialString(char *s)
{
	for (int i = 0; s[i] != '\0'; i++)
      if (!WriteSerialOneByte(s[i]))
           return 01;
    return 1;
}

// ***************************************************
//
// ***************************************************
int ReadSerialOneByte(void)
{
    unsigned char data=0;
    DWORD NumberOfBytesRead;

	if (!ReadFile(handle, &data, 1, &NumberOfBytesRead, NULL)) 
		return(-1);
    else 
		return((int) data);
}


// ***************************************************
//
// ***************************************************
void ReadStrFromSerial(char *data, int tam)
{
	int i;

	for (i=0; i<tam; i++)
	{
		data[i] = (char) ReadSerialOneByte();
	}
	data[i] = '\0';
}

// ***************************************************
//
// ***************************************************
void SerialClearBuffer()
{
	PurgeComm(handle, PURGE_RXCLEAR);
}
